@@ -5,21 +5,23 @@ use Mojo::IOLoop;
 use Mojo::IOLoop::Client;
 use Mojo::IOLoop::Delay;
 use Mojo::IOLoop::Stream;
+use Mojo::Util 'b64_encode';
 use Mojo::SMTP::Client::Response;
 use Mojo::SMTP::Client::Exception;
 use Scalar::Util 'weaken';
 use Carp;
 
-our $VERSION = 0.07;
+our $VERSION = 0.08;
 
 use constant {
 	CMD_OK       => 2,
 	CMD_MORE     => 3,
 	
 	CMD_CONNECT  => 1,
-	CMD_STARTTLS => 10,
 	CMD_EHLO     => 2,
 	CMD_HELO     => 3,
+	CMD_STARTTLS => 10,
+	CMD_AUTH     => 11,
 	CMD_FROM     => 4,
 	CMD_TO       => 5,
 	CMD_DATA     => 6,
@@ -30,9 +32,10 @@ use constant {
 
 our %CMD = (
 	&CMD_CONNECT  => 'CMD_CONNECT',
-	&CMD_STARTTLS => 'CMD_STARTTLS',
 	&CMD_EHLO     => 'CMD_EHLO',
 	&CMD_HELO     => 'CMD_HELO',
+	&CMD_STARTTLS => 'CMD_STARTTLS',
+	&CMD_AUTH     => 'CMD_AUTH',
 	&CMD_FROM     => 'CMD_FROM',
 	&CMD_TO       => 'CMD_TO',
 	&CMD_DATA     => 'CMD_DATA',
@@ -77,7 +80,8 @@ sub send {
 	
 	# user changed SMTP server or server sent smth while it shouldn't
 	if ($this->{stream} && (($this->{server} ne $this->_server) ||
-	     ($this->{stream}->is_readable && !$this->{starttls} && grep {$this->{last_cmd} == $_} (CMD_CONNECT, CMD_DATA_END, CMD_RESET)))
+	     ($this->{stream}->is_readable && !$this->{starttls} && !$this->{authorized} && 
+	      grep {$this->{last_cmd} == $_} (CMD_CONNECT, CMD_DATA_END, CMD_RESET)))
 	) {
 		$this->_rm_stream();
 	}
@@ -86,7 +90,7 @@ sub send {
 		push @steps, sub {
 			my $delay = shift;
 			# connect
-			$this->{starttls} = 0;
+			$this->{starttls} = $this->{authorized} = 0;
 			$this->emit('start');
 			$this->{server} = $this->_server;
 			$this->{last_cmd} = CMD_CONNECT;
@@ -199,9 +203,9 @@ sub send {
 					}
 					
 					return $loop->reactor->watch($sock, 1, 0)
-						if $IO::Socket::SSL::SSL_ERROR == IO::Socket::SSL::SSL_WANT_READ;
+						if $IO::Socket::SSL::SSL_ERROR == IO::Socket::SSL::SSL_WANT_READ();
 					return $loop->reactor->watch($sock, 0, 1)
-						if $IO::Socket::SSL::SSL_ERROR == IO::Socket::SSL::SSL_WANT_WRITE;
+						if $IO::Socket::SSL::SSL_ERROR == IO::Socket::SSL::SSL_WANT_WRITE();
 					
 				})->watch($sock, 0, 1);
 			},
@@ -215,6 +219,20 @@ sub send {
 				$delay->pass;
 			}
 		}
+		elsif ($cmd[$i] eq 'auth') { # AUTH
+			push @steps, sub {
+				my $delay = shift;
+				$this->_cmd('AUTH PLAIN '.b64_encode(join("\0", '', $cmd[$mi]->{login}, $cmd[$mi]->{password}), ''), CMD_AUTH);
+				$this->_read_response($delay->begin, $mi == $#cmd);
+				$expected_code = CMD_OK;
+			},
+			$resp_checker,
+			sub {
+				my $delay = shift;
+				$this->{authorized} = 1;
+				$delay->pass;
+			}
+		}
 		elsif ($cmd[$i] eq 'from') { # FROM
 			push @steps, sub {
 				my $delay = shift;
@@ -415,9 +433,9 @@ sub _read_response {
 
 sub _has_nl {
 	if (ref $_[0]) {
-		return ${$_[0]} =~ /\012$/;
+		return ${$_[0]} =~ /\015\012$/;
 	}
-	$_[0] =~ /\012$/;
+	$_[0] =~ /\015\012$/;
 }
 
 sub DESTROY {
@@ -603,6 +621,13 @@ attributes
 	$smtp->tls_ca('/etc/ssl/certs/ca-certificates.crt');
 	$smtp->send(starttls => 1);
 
+=item auth
+
+Authorize on SMTP server. Argument to this command should be reference to a hash with C<login> and C<password>
+keys. Only PLAIN authorization supported for now. You should authorize only once per session
+
+	$smtp->send(auth => {login => 'oleg', password => 'qwerty'});
+
 =item from
 
 From which email this message was sent. Value for this cammand should be a string with email
@@ -676,9 +701,10 @@ clients to make simultaneous sending.
 C<Mojo::SMTP::Client> has this non-importable constants
 
 	CMD_CONNECT  # client connected to SMTP server
-	CMD_STARTTLS # client sent STARTTLS command
 	CMD_EHLO     # client sent EHLO command
 	CMD_HELO     # client sent HELO command
+	CMD_STARTTLS # client sent STARTTLS command
+	CMD_AUTH     # client sent AUTH command
 	CMD_FROM     # client sent MAIL FROM command
 	CMD_TO       # client sent RCPT TO command
 	CMD_DATA     # client sent DATA command
@@ -731,13 +757,13 @@ available on CPAN. L<MIME::Lite> for example. With such modules we can get
 email as a string and send it with C<Mojo::SMTP::Client>
 
 	use MIME::Lite;
-	use Encode;
+	use Encode qw(encode decode);
 	
 	my $msg = MIME::Lite->new(
 		Type    => 'text',
 		From    => 'me@home.org',
 		To      => 'you@work.org',
-		Subject => Encode::encode('MIME-Header', '世界, 労働, 5月!'),
+		Subject => encode('MIME-Header', decode('utf-8', '世界, 労働, 5月!')),
 		Data    => 'Novosibirsk (Russian: Новосибирск; IPA: [nəvəsʲɪˈbʲirsk]) is the third most populous '.
 		           'city in Russia after Moscow and St. Petersburg and the most populous city in Asian Russia'
 	);
@@ -795,6 +821,35 @@ as argument, so it will be super easy to send our big email in memory-efficient
 		data => sub { $generator->get() }
 	);
 
+=head2 How to send message using public email services like Gmail
+
+Most such services provides access via SMTP in addition to web interface, but needs authorization. To protect your
+login and password most of them requires to start encrypted session (by upgrading plain connection with C<starttls>
+or by initial C<tls> connection). For example Gmail supports both this ways:
+
+	# make plain connection to port 25
+	my $smtp = Mojo::SMTP::Client->new(address => 'smtp.gmail.com');
+	# and upgrade it to TLS with starttls
+	$smtp->send(
+		starttls => 1,
+		auth => {login => $login, password => $password},
+		from => $from,
+		to   => $to,
+		data => $msg,
+		quit => 1
+	);
+	
+	# or make initial TLS connection to port 465
+	my $smtp = Mojo::SMTP::Client->new(address => 'smtp.gmail.com', tls => 1);
+	# no need to use starttls
+	$smtp->send(
+		auth => {login => $login, password => $password},
+		from => $from,
+		to   => $to,
+		data => $msg,
+		quit => 1
+	);
+
 =head2 How to send message directly, without using of MTAs such as sendmail, postfix, exim, ...
 
 Sometimes it is more suitable to send message directly to SMTP server of recipient. For example
@@ -877,7 +932,9 @@ on your domain and so on.
 
 =head1 SEE ALSO
 
-L<Mojo::SMTP::Client::Response>, L<Mojo::SMTP::Client::Exception>, L<Mojolicious>, L<Mojo::IOLoop>
+L<Mojo::SMTP::Client::Response>, L<Mojo::SMTP::Client::Exception>, L<Mojolicious>, L<Mojo::IOLoop>,
+L<RFC5321 (SMTP)|https://tools.ietf.org/html/rfc5321>, L<RFC3207 (STARTTLS)|https://tools.ietf.org/html/rfc3207>,
+L<RFC4616 (AUTH PLAIN)|https://tools.ietf.org/html/rfc4616>
 
 =head1 COPYRIGHT
 
